function vrplay(seq, vr_fig, vr_world, data)
% VRPLAY Play VRML animations for the Marine Visualization Toolbox.
%    VRPLAY(SEQUENCE, VRFIG, VRWORLD, DATA) interpolates the desired sample 
%    range and time step (specified in SEQUENCE) of the data values given 
%    in DATA. The interpolated data modifies corresponding nodes in VRWORLD 
%    and VRFIG.
%
% See also VRFIGURE, VRWORLD, VRNODE, DIRECTOR, CREATEVRML.
%
% Author:     Andreas Lund Danielsen
% Date:       10th November 2003
% Revisions:


% interpolate data, to achieve desired animation speed.
% time_step = 1/framesPerSecond implies that one second of simulation
% equals one second of animation. this player tries to achieve 5 fps, 
% thus time_step = 0.2

    % set fps
    fps_inv = seq.speed * 0.2;
    % smallest interpolation length
    min_t = inf;    

    % retrieve desired set of data, [t_start t_stop]
    % for all vessels
    for cv = 1:length(data)

        % simulation data, raw
        raw.t = data(cv).t(seq.start:seq.stop);
        raw.x = data(cv).x(seq.start:seq.stop,:);
        
        % add 2*pi to avoid interpolation of steps
        tmp_raw.x = raw.x;
        for sample = 1:size(raw.x,1)-1
            if ( sign(raw.x(sample,6)) ~= sign(raw.x(sample+1,6)) ) & (abs(raw.x(sample,6)) > 2)
                tmp_raw.x(sample+1:end,6) = sign(raw.x(sample,6))*2*pi + tmp_raw.x(sample+1:end,6);
            end
        end
        raw.x = tmp_raw.x;
        
        % interpolated data, fine
        fine(cv).t = [data(cv).t(seq.start) : fps_inv : data(cv).t(seq.stop)];
        
        % interpolate all states
        for state = 1:6
            tmp = raw.x(:, state);
            fine(cv).x(:, state) = pchip(raw.t, tmp, fine(cv).t);
        end % state
        
        % find the smallest interpolated states
        min_t = min(min_t, length(fine(cv).t));
                
%         % uncomment this section to see plots of interpolated data
%         figure(2);
%         % plot position data, x vs y
%         plot(raw.t, raw.x(:,6), 'b.');       % input data
%         hold on;
%         plot(fine(1).t, fine.x(:,6), 'r-');     % interpolated data
%         return;
          
    end % cv

% create handles to vr nodes
for cv = 1:length(data)
    node_name = ['vessel_' num2str(cv)];
    node_view = ['view_'   num2str(cv)];
    node_path = ['path_'   num2str(cv) '_ext'];
    vessel(cv).h   = vrnode(vr_world, node_name);
    view(cv).h     = vrnode(vr_world, node_view);
    path(cv).h     = vrnode(vr_world, node_path);
end

% open waitbar
wb_h = waitbar(0, ['Frame 0/' num2str(min_t) '...']);

% play animation!
for t = 2:min_t
    
    % start timer
    tic;
    
    % for all vessels
    for cv = 1:length(data)
        
        % set translation
        vessel(cv).h.translation = fine(cv).x(t, 1:3);
        view(cv).h.translation = fine(cv).x(t, 1:3);
        % set rotation
        [beta, epsilon] = euler2p(fine(cv).x(t, 4:6));
        vessel(cv).h.rotation = [beta epsilon];
        
        % is path constant?
        % or redraw path...
        if strcmp(seq.vessel(cv).path_start, 't')
            % path_start is char
            en = eval(seq.vessel(cv).path_end);
            len = length(data(1).x);
            sta = seq.start + round(t/length(fine(1).t) * length(raw.t)) - 1;
            path(cv).h.scale = seq.vessel(cv).path_radius * [zeros(sta,2); ...
                                                             ones(en-sta,2); ...
                                                             zeros(len-en,2)];
        elseif strcmp(seq.vessel(cv).path_end, 't')
            % path_end is char
            sta = eval(seq.vessel(cv).path_start);
            len = length(data(1).x);
            en = seq.start + round(t/length(fine(1).t) * length(raw.t)) - 1;
            path(cv).h.scale = seq.vessel(cv).path_radius * [zeros(sta,2); ...
                                                             ones(en-sta,2); ...
                                                             zeros(len-en,2)];
        else
            % do nothing, path is constant
        end            
           
    end % for (cv, current vessel)
    
    % refresh figure
    vrdrawnow;
    
    % display waitbar
    try
        % diaplay wait message and continue
        msg = ['Frame ' num2str(t) '/' num2str(min_t) '...'];
        waitbar(t/min_t, wb_h, msg);
    catch
        % user closed waitbar, return from funtcion
        return;
    end
    
    % wait?
    while toc < (fine(1).t(t) - fine(1).t(t-1))/seq.speed
        % loop...
    end
    
end

% close waitbar
close(wb_h);